#!/usr/bin/python3
# -*- coding:utf8 -*-

import os
import sys
import json
import os.path
import subprocess


class NFSHpc(object):

    def __init__(self):
        self.root = ""
        self.groups = []
        self.pkgs = {}

        self.load_config()

    def load_config(self):
        execfile=os.path.abspath(sys.argv[0])
        config=os.path.join(os.path.dirname(execfile), "hpcsoft-config.json")
        with open(config, 'r') as f:
            data = json.load(f)

        self.root = data["root"]
        groups = data["groups"]

        for group in groups:
            for group_name, pkgs in group.items():
                if not group_name in self.groups:
                    self.groups.append(group_name)
                for pkg in pkgs:
                    self.pkgs[pkg["module"]] = pkg
        #print(self.pkgs)

    def list_allpkgs(self):
        all_pkgs = {}
        for group in self.groups:
            all_pkgs[group] = []
        for module, info in self.pkgs.items():
            all_pkgs[info["group"]].append(module)

        for group_name, pkgs in all_pkgs.items():
            print("group: %s " % group_name)
            for pkg in pkgs:
                print("    %s" % pkg)
            print("")




    def add_avail_pkg(self, name, pkgs_added):
        if name in self.pkgs.keys():
            info = self.pkgs[name]
            path = info["path"]
            fullpath = os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), path)

            if fullpath is None or not os.path.exists(fullpath):
                print("%s not found" % name)
                sys.exit(1)

            if not os.path.exists(os.path.join(fullpath, "install.sh")):
                print("%s not found" % name)
                sys.exit(1)

            pkgs_added.append(name)
            print("%s depends on %s" %(name, info["deps"]))
            for dep in info["deps"]:
                self.add_avail_pkg(dep, pkgs_added)

    def pkg_install(self, pkg):
        print("============================   installing %s   ============================" % pkg)
        path = self.pkgs[pkg]["path"]
        fullpath = os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), path)

        cmd = "bash " + os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), "hpcsoft-scripts.sh") + " " + pkg + " " + self.root + " " + path + " " + os.path.abspath(sys.argv[0])
        status = os.system(cmd)
        if status != 0:
            print("failed to install %s" % pkg)
            sys.exit(1)

    def pkgs_install(self, pkgs):
        pkgs_added = []
        for pkg in pkgs:
            self.add_avail_pkg(pkg, pkgs_added)

        pkgs_added_uniq = []
        for pkg in pkgs_added[::-1]:
            if pkg not in pkgs_added_uniq:
                pkgs_added_uniq.append(pkg)

        print("These packages will be installed:")
        for pkg in pkgs_added_uniq:
            print("    %s" % pkg)

        for pkg in pkgs_added_uniq:
            self.pkg_install(pkg)


    def groups_install(self, groups):
        pkgs = []
        for pkg, info in self.pkgs.items():
            if info["group"] in groups:
                pkgs.append(pkg)
        self.pkgs_install(pkgs)

    def pkginfo(self, name):
        if name in self.pkgs.keys():
            for key, value in self.pkgs[name].items():
                print("%s: %s" % (key, value))



    def help(self):
        name = os.path.basename(os.path.abspath(sys.argv[0]))
        print('''A tool to build and install hpc softwares.

  %s list:  list all softwares in this tool.
  %s install <software1> <software2> ...:  build and install one or more softwares.
  %s groupinstall <group1> <group2> ...:  build and install all softwares in these groups.
  %s pkginfo <software>:  show information about a software.
''' % (name, name, name, name))


def main():
    hpc = NFSHpc()

    if len(sys.argv) == 1:
        hpc.help()
        sys.exit(0)

    if sys.argv[1] == "install":
        hpc.pkgs_install(sys.argv[2:])
    elif sys.argv[1] == "groupinstall":
        hpc.groups_install(sys.argv[2:])
    elif sys.argv[1] == "list":
        hpc.list_allpkgs()
    elif sys.argv[1] == "pkginfo":
        if len(sys.argv) != 3:
            hpc.help()
            sys.exit(0)
        hpc.pkginfo(sys.argv[2])
    else:
        hpc.help()


if __name__ == '__main__':
    main()

