#!/usr/bin/bash

# $1 module name
# $2 root directory
# $3 module path
# $4 exec directory


export MODULE_NAME=$1
export PKGROOT=$2
MODULE_PATH=$3
EXECFILE=$4
MODULE_BASENAME=`echo ${MODULE_NAME} | sed -e "s/\//-/g"`
LOGDIR=${PKGROOT}/.pkgs/log/${MODULE_BASENAME}

export MODULEPATH="${PKGROOT}/modules"
export LOGFILE=${LOGDIR}/${MODULE_BASENAME}.log
export PKGPATH="${PKGROOT}/${MODULE_PATH}"
export SRCPATH=`dirname ${EXECFILE}`/${MODULE_PATH}
export TMPDIR=/tmp/nfs-hpcsoft/${MODULE_BASENAME}.$$

MODULE_FILE=${MODULEPATH}/${MODULE_NAME}

_PKGPATH=$(echo ${PKGPATH} | sed -e "s/\//\\\\\//g")
_PKGROOT=$(echo ${PKGROOT} | sed -e "s/\//\\\\\//g")

die() {
    echo $1
#    rm -rf ${TMPDIR}
    rm -rf ${PKGPATH}
    rm -rf ${MODULE_FILE}
    exit 1
}
export -f die

check_pkg_install()
{
    pkg_install_status=0
    swlist="gcc gcc-c++ gcc-gfortran libXt-devel numactl-devel readline-devel tcl tcsh bzip2-devel xz-devel openssl-devel libcurl-devel pcre2-devel libpng-devel"
    need_install=()
    for install_pkg in ${swlist[@]};do
        rpm -q ${install_pkg} >/dev/null
        if [[ $? -ne 0 ]];then
            need_install+=(${install_pkg})            
            (( pkg_install_status++ ))
        fi
    done

    if [[ "${pkg_install_status}" -ne 0 ]];then
        echo "Please check the required installation package, total: ${pkg_install_status}"
        echo -e "\e[31m${need_install[@]}\e[0m needs to be installed, try \e[31mdnf install or yum install\e[0m"
        exit 1
    fi
}

# 检查依赖软件包是否已经安装
echo "step 0: Check if required packages are already installed on the system."
check_pkg_install

# 检查软件包是否已经安装
echo "step 1: Check if ${MODULE_NAME} is already installed on the system."
module is-avail ${MODULE_NAME}
if [ $? -eq 0 ]
then
        echo "${MODULE_NAME} already installed, no need to reinstall."
	exit 0
fi
echo ${MODULE_NAME} not installed, starting installation now.


# 创建工作目录
mkdir -p ${LOGDIR}
rm -rf ${LOGFILE}
touch ${LOGFILE}

rm -rf ${PKGPATH}
mkdir -p ${PKGPATH}

mkdir -p $TMPDIR


# 加载依赖的软件包
echo "step 2: Load the software packages required by ${MODULE_NAME}"
modules=`${EXECFILE} pkginfo ${MODULE_NAME} | grep "^deps:" | grep "^deps:"  | sed -e "s/deps: \[//g" | sed -e "s/]//g" | tr ',' ' ' | sed -e "s/'//g"`
module load ${modules}
if [ $? -ne 0 ]
then    
    echo "Error loading dependency packages, exit."
    exit 1
fi
export modules
echo "Successfully loaded dependency packages."


# 编译和安装软件包
echo "step 3: Begin compiling ${MODULE_NAME}"
bash ${SRCPATH}/install.sh
result=$?
if [ $result -ne 0 ]
then
    if [ $result -eq 100 ]
    then
        echo "no need install ${MODULE_NAME}, exit."
        exit 0
    else
        die "Compilation of ${MODULE_NAME} failed, exit."
    fi
fi
echo "Compilation of ${MODULE_NAME} successful."


# 创建module文件
echo "step 4: Create module file"
mkdir -p `dirname ${MODULE_FILE}`
cat ${SRCPATH}/module.sh | sed -e "s/__PKGPATH__/${_PKGPATH}/" | sed -e "s/__PKGROOT__/${_PKGROOT}/" > ${MODULE_FILE}
if [ $? -ne 0 ]
then
    die "Error creating module file, exit."
fi
echo "Successfully created module file ${MODULE_FILE}"

# 加载软件包
echo "step 5: Test ${MODULE_NAME}"
module load ${MODULE_NAME}
if [ $? -ne 0 ]
then
    die "failed to load ${MODULE_NAME}"
fi


# 测试
if [ -f ${SRCPATH}/test.sh ]
then
   bash ${SRCPATH}/test.sh
   if [ $? -ne 0 ]
   then
       die "Testing failed, exit."
   fi
fi
echo "Testing successful."

echo "${MODULE_NAME} install success!"
exit 0

