#!/usr/bin/bash

check_build_deps()
{
    rpm -qi numactl-devel > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
        die "numactl-devel is needed by openmpi, please install numactl-devel first."
    fi

    rpm -qi gcc-gfortran > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
        die "gcc-gfortran is needed by openmpi, please install gcc-gfortran first."
    fi

}

make_install_ucx()
{
    echo "configure ucx with gnu ..."
    rm -rf ${TMPDIR}/*
    mkdir -p ${PKGPATH}/external_libs/ucx/1.17.0
    tar --no-same-owner -xzf ${SRCPATH}/src/ucx-1.17.0.tar.gz -C ${TMPDIR} || die "ucx uncompression failed, see $LOGFILE for details."
    cd ${TMPDIR}/ucx-1.17.0
    ./configure --prefix=${PKGPATH}/external_libs/ucx/1.17.0 \
    --enable-optimizations --disable-logging --disable-debug --disable-assertions \
    --disable-params-check --enable-mt --without-xpmem --without-java --without-cuda \
    --without-rocm --enable-devel-headers CC=/usr/bin/gcc CXX=/usr/bin/g++ >>$LOGFILE 2>&1 || die "ucx configure failed, see $LOGFILE for details."
    make -j `grep cores /proc/cpuinfo | wc -l` >>$LOGFILE 2>&1 || die "ucx make failed, see $LOGFILE for details."
    make install >>$LOGFILE 2>&1 || die "ucx make install failed, see $LOGFILE for details."
    cd -
    rm -rf ${TMPDIR}/*
}


make_install_openmpi()
{
    rm -rf ${TMPDIR}/*
    tar --no-same-owner -xjf ${SRCPATH}/src/openmpi-4.1.7.tar.bz2 -C ${TMPDIR}

    if [ $? -ne 0 ]; then
        die "OpenMPI uncompression failed, see $LOGFILE for details."
    fi

    cd ${TMPDIR}/openmpi-4.1.7
    echo "configure openmpi with gnu ..."
    ./configure --prefix=${PKGPATH} --with-libevent=internal --with-platform=contrib/platform/mellanox/optimized --without-hcoll --with-ucx=${PKGPATH}/external_libs/ucx/1.17.0 --without-cuda --without-xpmem --enable-mpi-cxx --enable-mpi1-compatibility --enable-orterun-prefix-by-default --enable-dlopen CC=$CC CXX=$CXX FC=$FC CFLAGS=-fPIC CXXFLAGS=-fPIC FCFLAGS=-fPIC >>$LOGFILE 2>&1 || die "OpenMPI configure failed, see $LOGFILE for details."

    NP=${SYS_NP}
    echo "make openmpi ..."
    make -j `grep cores /proc/cpuinfo | wc -l` >>$LOGFILE 2>&1 || die "OpenMPI make failed, see $LOGFILE for details."
    echo "make install openmpi ..."
    make install >>$LOGFILE 2>&1 || die "OpenMPI make install failed, see $LOGFILE for details."
    cd - 2>&1 >/dev/null
    rm -rf ${TMPDIR}/*
}



make_lib_link()
{
    cd ${PKGPATH}/lib/
    ln -s libmpi.so.40.20.3 libmpi.so.1
    cd -
}


main()
{
	echo "Start to install $MODULE_NAME ......"
	CC=/usr/bin/gcc
	CXX=/usr/bin/g++
	F77=/usr/bin/gfortran
	FC=/usr/bin/gfortran
	check_build_deps
	make_install_ucx
    make_install_openmpi
	#make_lib_link
	echo "OpenMPI (with gnu compiler) installation ... OK"
}


main $@
